//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;

namespace CLRUnleashed
{
	class A
	{
		public virtual void Message() 
		{
			Console.WriteLine("This is a message from A");
		}
	}
	sealed class B: A
	{
		public override void Message() 
		{
			Console.WriteLine("This is a message from B");
		}
	}
	class C: A
	{
		// methodhideoverride.cs(21,15): warning CS0114: 'CLRUnleashed.C.Message()'
		// hides inherited member 'CLRUnleashed.A.Message()'. To make the current
		// member override that implementation, add the override keyword. Otherwise
		// add the new keyword.
		// public void Message() 
		public new void Message() 
		{
			Console.WriteLine("This is a message from C");
		}
	}
	// Uncomment the following to get an error message
	// MethodHideOverride.cs(31): 'CLRUnleashed.D' : cannot inherit from 
	// sealed class 'CLRUnleashed.B'
	// class D: B
	// {
	// 	public override void Message() 
	// 	{
	// 		Console.WriteLine("This is a message from D");
	// 	}
	// }

	/// <summary>
	/// Summary description for MethodHideOverride.
	/// </summary>
	class MethodHideOverride
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			// Basic inheritance
			Console.WriteLine("---- A method");
			A a = new A();
			a.Message();
			Console.WriteLine("---- B method");
			B b = new B();
			b.Message();
			Console.WriteLine("---- Cast B to A method");
			((A)b).Message();
			
			// Hide a method
			Console.WriteLine("---- C method");
			C c = new C();
			c.Message();
			Console.WriteLine("---- Cast C to A method");
			((A)c).Message();
		}
	}
}
