//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Collections;

namespace CLRUnleashed
{
	public class PointEventArgs : EventArgs
	{
		private Point point;
		public PointEventArgs(int x, int y)
		{
			point = new Point(x, y);
		}
		public Point Point
		{
			get
			{
				return point;
			}
			set
			{
				point = value;
			}
		}
	}

	public class Point : System.Collections.IHashCodeProvider
	{
		// An event that clients can use to be notified whenever the
		// elements of the list change:
		public event EventHandler Changed;
		private int _x;
		private int _y;
		// Invoke the Changed event; called whenever point changes:
		private void OnChanged(PointEventArgs e) 
		{
			if (Changed != null)
				Changed(this, e);
		}
		public Point(int x, int y)
		{
			_x = x;
			_y = y;
			Changed = null;
		}
		public int X
		{
			get
			{
				return _x;
			}
			set
			{
				_x = value;
				PointEventArgs e = new PointEventArgs(_x, _y);
				OnChanged(e);
			}
		}
		public int Y
		{
			get
			{
				return _y;
			}
			set
			{
				_y = value;
				PointEventArgs e = new PointEventArgs(_x, _y);
				OnChanged(e);
			}
		}
		public Point Coordinate
		{
			get
			{
				return this;
			}
			set
			{
				_x = value._x;
				_y = value._y;
				PointEventArgs e = new PointEventArgs(_x, _y);
				OnChanged(e);
			}
		}
		public override bool Equals(object o)
		{
			bool ret = true;
			try
			{
				Point target = (Point)o;
				ret = (_x == target._x &&
					_y == target._y);
			}
			catch(Exception)
			{
				// Expecting invalid cast if Equals is called 
				// with the wrong argument.
				// Note that if I use the as operator and compare 
				// the result for null I end up in a recursive
				// situation because I have chosen to reuse this
				// method in the != and == operators.
				ret = false;
			}
			return ret;
		}
		public static bool operator == (Point a, Point b)
		{
			return a.Equals(b);
		}
		public static bool operator != (Point a, Point b)
		{
			return !a.Equals(b);
		}
		public override int GetHashCode()
		{
			return _x ^ _y;
		}
		int IHashCodeProvider.GetHashCode(object o)
		{
			return o.GetHashCode();
		}
		public override string ToString()
		{
			return string.Format("{0},{1}", _x, _y);
		}
	}
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	class ReferencePoint
	{
		private static void Changed(object sender, EventArgs e) 
		{
			PointEventArgs pe = (PointEventArgs)e;
			Console.WriteLine("Point has changed. ({0}, {1})", pe.Point.X, pe.Point.Y);
		}
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			Point rp = new Point(0, 0);
			Console.WriteLine("Point {0}", rp.GetType().BaseType);
			rp.Changed += new EventHandler(Changed);
			rp.Coordinate = new Point(1, 2);
			Point rp1 = rp;
			Console.WriteLine("p Point == p1 Point-> {0}", rp == rp1);
			Console.WriteLine("p Point Equals(p1 Point)-> {0}", rp.Equals(rp1));
			rp1 = new Point(1, 2);
			Console.WriteLine("p Point == p1 Point-> {0}", rp == rp1);
			Console.WriteLine("p Point Equals(p1 Point)-> {0}", rp.Equals(rp1));
			Console.WriteLine("Point {0}", rp.ToString());
		}
	}
}
