//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Security;
using System.Security.Permissions;
using System.IO;

namespace CLRUnleashed
{
	// Default is to give FullTrust to all code run in this computer
	// If this is changed to Everything which is all of the known
	// built-in permissions then this custom permission is ignored
	// and not granted then there will be a security exception.
	[CustomPermissionAttribute(SecurityAction.InheritanceDemand)]
	public class MyClass
	{
		public MyClass()
		{    
		}   

		public virtual void Message()
		{
			Console.WriteLine("This is a message from MyClass");
		}
	}
	public class YourClass : MyClass
	{
		public override void Message()
		{
			Console.WriteLine("This is a message from YourClass");
		}
	}
	/// <summary>
	/// Summary description for SecurityInheritance.
	/// </summary>
	class SecurityInheritance
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			CustomPermission perm = 
				new CustomPermission(PermissionState.Unrestricted);
			NamedPermissionSet pset = 
				new NamedPermissionSet("MyPermissionSet", PermissionState.None);
			pset.Description = "Permission set containing my custom permission";
			pset.AddPermission(perm);
			StreamWriter file = new StreamWriter("mypermissionset.xml");
			file.Write(pset.ToXml());
			file.Close();
			MyClass mc = new MyClass();
			mc.Message();
			YourClass yc = new YourClass();
			yc.Message();
		}
	}
}
